"""Module for the base data source class"""

import hashlib
import json
import logging
from pathlib import Path

from evidence.schemas import Response, SourceDataType

_logger = logging.getLogger(__name__)


class DataSource:
    """A base class for data sources"""

    @staticmethod
    def format_response(resp: Response) -> Response:
        """Add `id` to resp object if data exists

        :param Response resp: Response object
        :return: Response object with `id` field added if data exists
        """
        if resp.data:
            blob = json.dumps(
                resp.model_dump(), sort_keys=True, separators=(",", ":"), indent=None
            ).encode("utf-8")
            digest = hashlib.md5(blob)  # noqa: S324
            resp.id = f"normalize.evidence:{digest.hexdigest()}"
        return resp


class DownloadableDataSource(DataSource):
    """A base class for sources that use downloadable data"""

    def __init__(
        self, data_url: str, src_dir_path: Path, ignore_transformed_data: bool
    ) -> None:
        """Initialize DownloadableDataSource class

        :param str data_url: URL to data file
        :param Path src_dir_path: Path to source data directory
        :param bool ignored_transformed_data: `True` if only bare init is needed. This
            is intended for developers when using the CLI to transform source data.
            `False` will load the transformed data from s3
        """
        self.data_url = data_url
        self.src_dir_path = src_dir_path
        self.src_dir_path.mkdir(exist_ok=True, parents=True)
        self.ignore_transformed_data = ignore_transformed_data

    def download_s3_data(self, src_data_type: SourceDataType) -> Path:
        """Download data from public s3 bucket if it does not already exist in data
        directory and set the corresponding data path

        :param SourceDataType src_data_type: The data type contained in the
            transformed data file
        """
        raise NotImplementedError

    def get_transformed_data_path(
        self, transformed_data_path: Path, src_data_type: SourceDataType
    ) -> Path | None:
        """Get transformed data path for source

        :param Path transformed_data_path: The path to the transformed data file
        :param SourceDataType src_data_type: The data type contained in the
            transformed data file
        :return: Path to transformed data file
        """
        data_path = None
        if not self.ignore_transformed_data:
            if transformed_data_path:
                if transformed_data_path.exists():
                    data_path = transformed_data_path
                else:
                    _logger.error(
                        "The supplied path at %s does not exist.", transformed_data_path
                    )
            else:
                data_path = self.download_s3_data(src_data_type)
        return data_path



"""Module for accessing python client for cBioPortal."""

import csv
import logging
import shutil
from pathlib import Path

import boto3

from evidence import DATA_DIR_PATH
from evidence.data_sources.base import DownloadableDataSource
from evidence.schemas import Response, SourceDataType, SourceMeta, Sources

_logger = logging.getLogger(__name__)


class CBioPortal(DownloadableDataSource):
    """cBioPortal class."""

    def __init__(
        self,
        data_url: str = "https://cbioportal-datahub.s3.amazonaws.com/msk_impact_2017.tar.gz",
        src_dir_path: Path = DATA_DIR_PATH / "cbioportal",
        transformed_mutations_data_path: Path | None = None,
        transformed_case_lists_data_path: Path | None = None,
        ignore_transformed_data: bool = False,
    ) -> None:
        """Initialize cbioportal class

        :param str data_url: URL to data file
        :param Path src_dir_path: Path to cbioportal data directory
        :param Optional[Path] transformed_mutations_data_path: Path to transformed
            cbioportal mutations file
        :param Optional[Path] transformed_case_lists_data_path: Path to transformed
            cbioportal case_lists file
        :param bool ignore_transformed_data: `True` if only bare init is needed. This
            is intended for developers when using the CLI to transform cbioportal data.
            Ignores path set in `transformed_mutations_data_path` and
            `transformed_case_lists_data_path`. `False` will load transformed
            data from s3
        """
        super().__init__(data_url, src_dir_path, ignore_transformed_data)
        self.source_meta = SourceMeta(
            label=Sources.CBIOPORTAL, version="msk_impact_2017"
        )
        self.transformed_mutations_data_path = self.get_transformed_data_path(
            transformed_mutations_data_path, SourceDataType.CBIOPORTAL_MUTATIONS
        )
        self.transformed_case_lists_data_path = self.get_transformed_data_path(
            transformed_case_lists_data_path, SourceDataType.CBIOPORTAL_CASE_LISTS
        )

    def download_s3_data(
        self, src_data_type: SourceDataType = SourceDataType.CBIOPORTAL_MUTATIONS
    ) -> Path:
        """Download MSK Impact 2017 mutation and case_lists data from public s3 bucket
        if it does not already exist in data directory and set the corresponding
        data path

        :param SourceDataType src_data_type: The data type contained in the
            transformed data file
        :return: Path to transformed data file
        """
        is_mutations = src_data_type == SourceDataType.CBIOPORTAL_MUTATIONS
        data_type = "mutations" if is_mutations else "case_lists"
        _logger.info("Retrieving transformed %s data from s3 bucket...", data_type)
        s3 = boto3.client("s3")
        zip_fn = (
            "msk_impact_2017_mutations.csv.zip"
            if is_mutations
            else "msk_impact_2017_case_lists.csv.zip"
        )
        zip_path = self.src_dir_path / zip_fn
        with zip_path.open("wb") as f:
            s3.download_fileobj(
                "vicc-normalizers", f"evidence_normalization/cbioportal/{zip_fn}", f
            )
        shutil.unpack_archive(zip_path, self.src_dir_path)
        Path.unlink(zip_path)
        _logger.info(
            "Successfully downloaded transformed cBioPortal %s data", data_type
        )
        return self.src_dir_path / zip_fn[:-4]

    def cancer_types_summary(self, hgnc_symbol: str) -> Response:
        """Get cancer types with gene mutations data

        :param str hgnc_symbol: HGNC symbol
        :return: Cancer types summary for gene
        """
        hgnc_symbol = hgnc_symbol.upper()

        mutation_sample_ids = set()
        with self.transformed_mutations_data_path.open() as f:
            data = csv.reader(f)
            headers = next(data)
            for row in data:
                if row[headers.index("Hugo_Symbol")] == hgnc_symbol:
                    sample_id = row[headers.index("Tumor_Sample_Barcode")]
                    mutation_sample_ids.add(sample_id)

        if not mutation_sample_ids:
            return self.format_response(
                Response(data={}, source_meta_=self.source_meta)
            )

        tumor_type_totals = {}
        with self.transformed_case_lists_data_path.open() as f:
            data = csv.reader(f)
            headers = next(data)
            for row in data:
                case_list_name = row[headers.index("case_list_name")]
                if ":" in case_list_name:
                    tumor_type = case_list_name.split(": ")[-1]
                    sample_ids = row[headers.index("case_list_ids")].split("\t")
                    tumor_type_totals[tumor_type] = {
                        "count": 0,
                        "total": len(sample_ids),
                    }
                    for sample_id in sample_ids:
                        if sample_id in mutation_sample_ids:
                            tumor_type_totals[tumor_type]["count"] += 1
                    tumor_type_totals[tumor_type]["percent_altered"] = (
                        tumor_type_totals[tumor_type]["count"]
                        / tumor_type_totals[tumor_type]["total"]
                    ) * 100
        return self.format_response(
            Response(data=tumor_type_totals, source_meta_=self.source_meta)
        )



"""Provide utilities for tests."""

from pathlib import Path

import pytest


@pytest.fixture(scope="session")
def evidence_data_dir() -> Path:
    """Return evidence data directory"""
    return Path(__file__).resolve().parents[1] / "evidence" / "data"



"""Module for ETL cbioportal data"""

import logging
import tarfile
import warnings
from pathlib import Path

import pandas as pd
import requests

from evidence import DATA_DIR_PATH
from evidence.data_sources import CBioPortal
from evidence.dev.etl import ETL_DATA_DIR_PATH

warnings.filterwarnings("ignore")
_logger = logging.getLogger(__name__)


class CBioPortalETLError(Exception):
    """Exceptions for CBioPortal ETL"""


class CBioPortalETL(CBioPortal):
    """Class for cBioPortal ETL methods."""

    def __init__(
        self,
        data_url: str = "https://cbioportal-datahub.s3.amazonaws.com/msk_impact_2017.tar.gz",
        src_dir_path: Path = DATA_DIR_PATH / "cbioportal",
        transformed_mutations_data_path: Path | None = None,
        transformed_case_lists_data_path: Path | None = None,
        ignore_transformed_data: bool = True,
    ) -> None:
        """Initialize cbioportal etl class

        :param str data_url: URL to data file
        :param Path src_dir_path: Path to cbioportal data directory
        :param Optional[Path] transformed_mutations_data_path: Path to transformed
            cbioportal mutations file
        :param Optional[Path] transformed_case_lists_data_path: Path to transformed
            cbioportal case_lists file
        :param bool ignore_transformed_data: `True` if only bare init is needed. This
            is intended for developers when using the CLI to transform cbioportal data.
            Ignores paths set in `transformed_mutations_data_path` and
            `transformed_case_lists_data_path`. `False` will load transformed
            data from s3
        """
        super().__init__(
            data_url,
            src_dir_path,
            transformed_mutations_data_path,
            transformed_case_lists_data_path,
            ignore_transformed_data,
        )
        self.src_dir_etl_path = ETL_DATA_DIR_PATH / "cbioportal"
        self.src_dir_etl_path.mkdir(exist_ok=True, parents=True)
        self.msk_impact_2017_dir = None

    def download_data(self) -> None:
        """Download MSK Impact 2017 data."""
        response = requests.get(self.data_url, stream=True, timeout=5)
        if response.status_code == 200:
            file = tarfile.open(fileobj=response.raw, mode="r|gz")
            file.extractall(path=self.src_dir_etl_path)  # noqa: S202
            self.msk_impact_2017_dir = self.src_dir_etl_path / "msk_impact_2017"
        else:
            _logger.error(
                "Unable to download cBioPortal data. Received status code: %i",
                response.status_code,
            )

    def transform_data(self) -> None:
        """Transform cbioportal data and write to csv files"""
        if not self.msk_impact_2017_dir:
            self.download_data()

        if not self.msk_impact_2017_dir:
            err_msg = "Downloading cBioPortal data was unsuccessful"
            raise CBioPortalETLError(err_msg)

        case_lists_df = self.create_case_lists_df()
        mutations_df = self.create_mutations_df()

        case_lists_data_path = self.src_dir_path / "msk_impact_2017_case_lists.csv"
        mutations_data_path = self.src_dir_path / "msk_impact_2017_mutations.csv"
        case_lists_df.to_csv(case_lists_data_path)
        mutations_df.to_csv(mutations_data_path)
        _logger.info("Successfully transformed cBioPortal data.")

    def create_case_lists_df(self) -> pd.DataFrame:
        """Create case lists data frame

        :return: Dataframe containing case list data
        """
        pathlist = Path(f"{self.msk_impact_2017_dir}/case_lists").glob("case_list_*")
        frames = []
        for p in pathlist:
            tmp = pd.read_csv(str(p), sep="delimiter", header=None)
            tmp = tmp[0].str.split(":", n=1, expand=True)
            tmp = tmp.set_index(0).T
            del tmp["cancer_study_identifier"]
            frames.append(tmp)
        return pd.concat(frames, ignore_index=True)

    def create_mutations_df(self) -> pd.DataFrame:
        """Create mutations data frame

        :return: Dataframe containing mutation data
        """
        return pd.read_csv(
            f"{self.msk_impact_2017_dir}/data_mutations.txt", sep="\t", skiprows=1
        )



"""Module containing schemas"""

from enum import Enum

from pydantic import BaseModel, ConfigDict, Field, StrictStr


class Base(BaseModel, extra="forbid"):
    """Base class for pydantic models"""


class Sources(str, Enum):
    """Define data sources"""

    CANCER_HOTSPOTS = "Cancer Hotspots"
    CBIOPORTAL = "cBioPortal"


class SourceMeta(Base):
    """Metadata for sources"""

    label: Sources
    version: StrictStr | None = None


class Response(Base):
    """Response model"""

    model_config = ConfigDict(populate_by_name=True)

    id: str | None = Field(default=None, alias="_id")
    data: dict
    source_meta_: SourceMeta


class SourceDataType(str, Enum):
    """Define constraints for source data type for data file"""

    CANCER_HOTSPOTS = "mutation_hotspots"
    CBIOPORTAL_CASE_LISTS = "case_lists"
    CBIOPORTAL_MUTATIONS = "mutations"



"""Module for testing cbioportal"""

import pytest

from evidence.data_sources import CBioPortal


@pytest.fixture(scope="module")
def cbioportal(evidence_data_dir):
    """Create test fixture for cbioportal class"""
    cbioportal_data_dir = evidence_data_dir / "cbioportal"
    return CBioPortal(
        transformed_mutations_data_path=cbioportal_data_dir
        / "msk_impact_2017_mutations.csv",
        transformed_case_lists_data_path=cbioportal_data_dir
        / "msk_impact_2017_case_lists.csv",
    )


@pytest.fixture(scope="module")
def braf():
    """Create test fixture for braf"""
    return {
        "Colorectal Cancer": {
            "count": 119,
            "total": 1006,
            "percent_altered": 119 / 1006 * 100,
        },
        "Anal Cancer": {"count": 1, "total": 32, "percent_altered": 1 / 32 * 100},
        "Skin Cancer, Non-Melanoma": {
            "count": 4,
            "total": 148,
            "percent_altered": 4 / 148 * 100,
        },
        "Miscellaneous Brain Tumor": {"count": 0, "total": 6, "percent_altered": 0.0},
        "Bladder Cancer": {
            "count": 17,
            "total": 423,
            "percent_altered": 17 / 423 * 100,
        },
        "Small Bowel Cancer": {
            "count": 6,
            "total": 35,
            "percent_altered": 6 / 35 * 100,
        },
        "Renal Cell Carcinoma": {"count": 0, "total": 360, "percent_altered": 0.0},
        "Uterine Sarcoma": {"count": 2, "total": 93, "percent_altered": 2 / 93 * 100},
        "Embryonal Tumor": {  # Peripheral Nervous System on cBioPortal UI
            "count": 2,
            "total": 86,
            "percent_altered": 2 / 86 * 100,
        },
        "Retinoblastoma": {"count": 0, "total": 4, "percent_altered": 0.0},
        "Endometrial Cancer": {
            "count": 9,
            "total": 218,
            "percent_altered": 9 / 218 * 100,
        },
        "Glioma": {"count": 17, "total": 553, "percent_altered": 17 / 553 * 100},
        "Melanoma": {"count": 114, "total": 350, "percent_altered": 114 / 350 * 100},
        "Cervical Cancer": {"count": 1, "total": 50, "percent_altered": 1 / 50 * 100},
        "Soft Tissue Sarcoma": {
            "count": 6,
            "total": 435,
            "percent_altered": 6 / 435 * 100,
        },
        "NA": {"count": 1, "total": 57, "percent_altered": 1 / 57 * 100},
        "Multiple Myeloma": {"count": 0, "total": 1, "percent_altered": 0},
        "Small Cell Lung Cancer": {
            "count": 3,
            "total": 82,
            "percent_altered": 3 / 82 * 100,
        },
        "Miscellaneous Neuroepithelial Tumor": {
            "count": 0,
            "total": 9,
            "percent_altered": 0.0,
        },
        "Pancreatic Cancer": {
            "count": 13,
            "total": 502,
            "percent_altered": 13 / 502 * 100,
        },
        "Salivary Gland Cancer": {
            "count": 2,
            "total": 114,
            "percent_altered": 2 / 114 * 100,
        },
        "Penile Cancer": {"count": 1, "total": 7, "percent_altered": 1 / 7 * 100},
        "Hodgkin Lymphoma": {"count": 0, "total": 5, "percent_altered": 0.0},
        "Pheochromocytoma": {"count": 0, "total": 4, "percent_altered": 0.0},
        "Breast Cancer": {"count": 4, "total": 1324, "percent_altered": 4 / 1324 * 100},
        "Breast Sarcoma": {"count": 0, "total": 13, "percent_altered": 0.0},
        "Adrenocortical Carcinoma": {
            "count": 0,
            "total": 26,  # UI says 25, but API combines Adrenocortical Adenoma (1) and  Adrenocortical Carcinoma (25), making it 26
            "percent_altered": 0.0,
        },
        "Germ Cell Tumor": {"count": 2, "total": 288, "percent_altered": 2 / 288 * 100},
        "Head and Neck Cancer": {
            "count": 1,
            "total": 173,
            "percent_altered": 1 / 173 * 100,
        },
        "Hepatobiliary Cancer": {
            "count": 11,
            "total": 338,
            "percent_altered": 11 / 338 * 100,
        },
        "Esophagogastric Cancer": {
            "count": 5,
            "total": 341,
            "percent_altered": 5 / 341 * 100,
        },
        "Prostate Cancer": {
            "count": 12,
            "total": 717,
            "percent_altered": 12 / 717 * 100,
        },
        "Ampullary Carcinoma": {
            "count": 0,
            "total": 25,  # UI says 9, but API combines Ampullary Cancer (16) and Ampullary Carcinoma (9), making it 25
            "percent_altered": 0.0,
        },
        "Histiocytosis": {"count": 6, "total": 22, "percent_altered": 6 / 22 * 100},
        "Gastrointestinal Neuroendocrine Tumor": {
            "count": 4,
            "total": 46,
            "percent_altered": 4 / 46 * 100,
        },
        "Non-Small Cell Lung Cancer": {
            "count": 87,
            "total": 1668,
            "percent_altered": 87 / 1668 * 100,
        },
        "Non-Hodgkin Lymphoma": {
            "count": 5,
            "total": 165,
            "percent_altered": 5 / 165 * 100,
        },
        "Cancer of Unknown Primary": {
            "count": 10,
            "total": 184,
            "percent_altered": 10 / 184 * 100,
        },
        "Mesothelioma": {"count": 0, "total": 107, "percent_altered": 0.0},
        "Thyroid Cancer": {
            "count": 88,
            "total": 231,
            "percent_altered": 88 / 231 * 100,
        },
        "Pineal Tumor": {"count": 0, "total": 3, "percent_altered": 0.0},
        "Wilms Tumor": {"count": 0, "total": 2, "percent_altered": 0.0},
        "Thymic Tumor": {"count": 0, "total": 18, "percent_altered": 0.0},
        "Appendiceal Cancer": {
            "count": 1,
            "total": 79,
            "percent_altered": 1 / 79 * 100,
        },
        "Gestational Trophoblastic Disease": {
            "count": 0,
            "total": 3,
            "percent_altered": 0.0,
        },
        "Leukemia": {"count": 1, "total": 4, "percent_altered": 1 / 4 * 100},
        "Mastocytosis": {"count": 0, "total": 1, "percent_altered": 0.0},
        "Vaginal Cancer": {"count": 0, "total": 4, "percent_altered": 0.0},
        "Sex Cord Stromal Tumor": {"count": 0, "total": 19, "percent_altered": 0.0},
        "Nerve Sheath Tumor": {"count": 0, "total": 16, "percent_altered": 0.0},
        "Sellar Tumor": {"count": 0, "total": 5, "percent_altered": 0.0},
        "Bone Cancer": {"count": 1, "total": 134, "percent_altered": 1 / 134 * 100},
        "Gastrointestinal Stromal Tumor": {
            "count": 2,
            "total": 137,
            "percent_altered": 2 / 137 * 100,
        },
        "CNS Cancer": {"count": 1, "total": 48, "percent_altered": 1 / 48 * 100},
        "Ovarian Cancer": {"count": 3, "total": 224, "percent_altered": 3 / 224 * 100},
    }


def test_get_mutation_data(cbioportal, braf):
    """Test that get_mutation_data method works correctly."""
    resp = cbioportal.cancer_types_summary("braf").model_dump(by_alias=True)
    data = resp["data"]
    assert data.keys() == braf.keys()

    total_cancer_type_count = sum(v["count"] for v in data.values())
    assert total_cancer_type_count == 562

    assert data == braf
    assert resp["_id"] == "normalize.evidence:0d76601b070d3c9f36e9c483454adf48"
    assert resp["source_meta_"]["label"] == "cBioPortal"
    assert resp["source_meta_"]["version"] == "msk_impact_2017"

    resp = cbioportal.cancer_types_summary("dummy").model_dump()
    assert resp["data"] == {}
    assert resp["source_meta_"]["label"] == "cBioPortal"
    assert resp["source_meta_"]["version"] == "msk_impact_2017"



"""Module for getting cancer hotspots data.

Downloads Site: https://www.cancerhotspots.org/#/download
Data URL: https://www.cancerhotspots.org/files/hotspots_v2.xls
"""

import json
import logging
import shutil
from pathlib import Path

import boto3
from botocore.config import Config

from evidence import DATA_DIR_PATH
from evidence.data_sources.base import DownloadableDataSource
from evidence.schemas import Response, SourceDataType, SourceMeta, Sources

_logger = logging.getLogger(__name__)


class CancerHotspots(DownloadableDataSource):
    """Class for Cancer Hotspots Data Access."""

    def __init__(
        self,
        data_url: str = "https://www.cancerhotspots.org/files/hotspots_v2.xls",
        src_dir_path: Path = DATA_DIR_PATH / "cancer_hotspots",
        transformed_data_path: Path | None = None,
        ignore_transformed_data: bool = False,
    ) -> None:
        """Initialize Cancer Hotspots class

        :param data_url: URL to data file
        :param src_dir_path: Path to cancer hotspots data directory
        :param transformed_data_path: Path to transformed cancer hotspots file
        :param ignore_transformed_data: `True` if only bare init is needed. This is
            intended for developers when using the CLI to transform cancer hotspots
            data. Ignores path set in `_transformed_data_path`. `False` will load
            transformed data from s3
        """
        super().__init__(data_url, src_dir_path, ignore_transformed_data)

        self.source_meta = SourceMeta(label=Sources.CANCER_HOTSPOTS, version="2")
        transformed_data_path = self.get_transformed_data_path(
            transformed_data_path, SourceDataType.CANCER_HOTSPOTS
        )
        if transformed_data_path:
            with transformed_data_path.open("r") as f:
                self.transformed_data = json.load(f)
        else:
            self.transformed_data = {}

    def download_s3_data(
        self, src_data_type: SourceDataType = SourceDataType.CANCER_HOTSPOTS
    ) -> None:
        """Download Cancer Hotspots data from public s3 bucket if it
        does not already exist in data directory and set the corresponding data path

        :param src_data_type: The data type contained in the transformed data file
        :return: Path to transformed data file
        """
        data_path = None
        _logger.info("Retrieving transformed cancer hotspots data from s3 bucket...")
        s3 = boto3.resource("s3", config=Config(region_name="us-east-2"))
        prefix = f"evidence_normalization/cancer_hotspots/{src_data_type.value}_"
        bucket = sorted(
            s3.Bucket("vicc-normalizers").objects.filter(Prefix=prefix).all(),
            key=lambda o: o.key,
        )
        if len(bucket) > 0:
            obj = bucket.pop().Object()
            obj_s3_path = obj.key
            zip_fn = obj_s3_path.split("/")[-1]
            fn = zip_fn[:-4]
            transformed_data_path = self.src_dir_path / fn
            if not transformed_data_path.exists():
                zip_path = self.src_dir_path / zip_fn
                with zip_path.open("wb") as f:
                    obj.download_fileobj(f)
                shutil.unpack_archive(zip_path, self.src_dir_path)
                Path.unlink(zip_path)
                _logger.info("Successfully downloaded transformed Cancer Hotspots data")
            else:
                _logger.info("Latest transformed Cancer Hotspots data already exists")

            data_path = transformed_data_path
        else:
            _logger.warning(
                "Could not find transformed Cancer Hotspots data in vicc-normalizers "
                "s3 bucket"
            )
        return data_path

    def mutation_hotspots(self, vrs_variation_id: str) -> Response:
        """Get cancer hotspot data for a variant

        :param vrs_variation_id: The VRS digest for the variation
        :return: Mutation hotspots data for variation
        """
        return self.format_response(
            Response(
                data=self.transformed_data.get(vrs_variation_id, {}),
                source_meta_=self.source_meta,
            )
        )



"""Module for initializing ETL classes"""

from os import environ
from pathlib import Path

ETL_PATH = Path(__file__).resolve().parents[0]
ETL_DATA_DIR_PATH = environ.get("ETL_DATA_DIR_PATH", ETL_PATH / "data")



"""Dev CLI"""

import logging

import asyncclick as click

from evidence.dev.etl.cancer_hotspots import CancerHotspotsETL, CancerHotspotsETLError
from evidence.dev.etl.cbioportal import CBioPortalETL, CBioPortalETLError


def _configure_logging() -> None:
    """Configure logging."""
    logging.basicConfig(
        filename=f"{__package__}.log",
        format="[%(asctime)s] - %(name)s - %(levelname)s : %(message)s",
    )
    logging.getLogger(__package__).setLevel(logging.DEBUG)


@click.command()
@click.option(
    "--transform_cancer_hotspots",
    is_flag=True,
    default=False,
    help="Transform Cancer Hotspots data",
)
@click.option(
    "--transform_cbioportal",
    is_flag=True,
    default=False,
    help="Transform cBioPortal data",
)
@click.option(
    "--transform_all",
    is_flag=True,
    default=False,
    help="Transforms all source data, currently cBioPortal and Cancer Hotspots",
)
async def cli(
    transform_cancer_hotspots: bool, transform_cbioportal: bool, transform_all: bool
) -> None:
    """Execute CLI methods

    :param bool transform_cancer_hotspots: Determines whether or not to transform
        Cancer Hotspots data
    :param bool transform_cbioportal: Determines whether or not to transform cBioPortal
        data
    :param bool transform_all: Transforms all source data
    """
    _configure_logging()
    if transform_all:
        transform_cbioportal_data()
        await transform_cancer_hotspots_data()
    else:
        if transform_cbioportal:
            transform_cbioportal_data()
        if transform_cancer_hotspots:
            await transform_cancer_hotspots_data()


def transform_cbioportal_data() -> None:
    """Transform cBioPortal data"""
    c = CBioPortalETL()
    try:
        c.transform_data()
    except CBioPortalETLError as e:
        click.echo(e)


async def transform_cancer_hotspots_data() -> None:
    """Transform Cancer Hotspots data"""
    c = CancerHotspotsETL()
    try:
        await c.add_vrs_identifier_to_data()
    except CancerHotspotsETLError as e:
        click.echo(e)


if __name__ == "__main__":
    cli(_anyio_backend="asyncio")



"""The VICC library for normalizing evidence"""

from os import environ
from pathlib import Path

APP_ROOT = Path(__file__).resolve().parents[0]
DATA_DIR_PATH = environ.get("DATA_DIR_PATH", APP_ROOT / "data")



"""Module for testing cancer hotspots"""

import pytest

from evidence.data_sources import CancerHotspots


@pytest.fixture(scope="module")
def cancer_hotspots(evidence_data_dir):
    """Create test fixture for cancer hotspots class"""
    globbed = (evidence_data_dir / "cancer_hotspots").glob("cancer_hotspots_*.json")
    return CancerHotspots(transformed_data_path=sorted(globbed)[-1])


@pytest.fixture(scope="module")
def braf_v600e():
    """Create test fixture for braf v600e"""
    return {
        "variation": "BRAF V600E",
        "codon": "V600",
        "mutation": "V600E",
        "q_value": 0.0,
        "observations": 833,
        "total_observations": 897,
    }


def check_source_meta(response):
    """Check that source meta is correct"""
    assert response["source_meta_"]["label"] == "Cancer Hotspots"
    assert response["source_meta_"]["version"] == "2"


def test_mutation_hotspots(cancer_hotspots, braf_v600e):
    """Test that mutation_hotspots method works correctly."""
    resp = cancer_hotspots.mutation_hotspots(
        "ga4gh:VA.urVNVupVvzqE57gZFBu6vsAFScIBNojG"
    )
    assert resp.data == {
        "variation": "BRAF N486_A489delinsK",
        "codon": "486-494",
        "mutation": "N486_A489delinsK",
        "q_value": 3.9500653726776106e-09,
        "observations": 1,
        "total_observations": 7,
    }

    resp = cancer_hotspots.mutation_hotspots(
        "ga4gh:VA.iKBIyhCE-3Cnh5XJpVNZ4t0Ju8Ov8Ine"
    )
    assert resp.data == {
        "variation": "BRAF N486_P490del",
        "codon": "486-494",
        "mutation": "N486_P490del",
        "q_value": 3.9500653726776106e-09,
        "observations": 3,
        "total_observations": 7,
    }

    resp = cancer_hotspots.mutation_hotspots(
        "ga4gh:VA.uIzxca0MnfZ1ni3NYNkvNh6Isw-5QXG9"
    )
    assert resp.data == {
        "variation": "TP53 I255del",
        "codon": "229-292",
        "mutation": "I255del",
        "q_value": 1.46521691051924e-64,
        "observations": 9,
        "total_observations": 76,
    }

    resp = cancer_hotspots.mutation_hotspots(
        "ga4gh:VA.AM5oy_X7xBxJmWyUPJ0ZSLpRaTgu2kHt"
    )
    assert resp.data == {
        "variation": "CEBPA T310_Q311insKQNP",
        "codon": "307-311",
        "mutation": "T310_Q311insKQNP",
        "q_value": 0.0050613902181432,
        "observations": 1,
        "total_observations": 4,
    }

    resp = cancer_hotspots.mutation_hotspots(
        vrs_variation_id="ga4gh:VA.j4XnsLZcdzDIYa5pvvXM7t1wn9OITr0L"
    ).model_dump(by_alias=True)
    assert resp["_id"] == "normalize.evidence:92f3db383a79d855323a71d65d860ec3"
    assert resp["data"] == braf_v600e
    check_source_meta(resp)

    # invalid vrs_variation_id
    resp = cancer_hotspots.mutation_hotspots(
        vrs_variation_id="ga4ghVA8JkgnqIgYqufNl-OV_hpRG_aWF9UFQCE"
    ).model_dump(by_alias=True)
    assert resp["_id"] is None
    assert resp["data"] == {}
    check_source_meta(resp)



"""Module for ETL cancer hotspots data"""

import datetime
import json
import logging
from pathlib import Path
from timeit import default_timer as timer

import pandas as pd
import requests
from variation.query import QueryHandler

from evidence import DATA_DIR_PATH
from evidence.data_sources import CancerHotspots


class CancerHotspotsETLError(Exception):
    """Exceptions for Cancer Hotspots ETL"""


_logger = logging.getLogger(__name__)


class CancerHotspotsETL(CancerHotspots):
    """Class for Cancer Hotspots ETL methods."""

    def __init__(
        self,
        data_url: str = "https://www.cancerhotspots.org/files/hotspots_v2.xls",
        src_dir_path: Path = DATA_DIR_PATH / "cancer_hotspots",
        transformed_data_path: Path | None = DATA_DIR_PATH / "cancer_hotspots",
        ignore_transformed_data: bool = True,
    ) -> None:
        """Initialize the CancerHotspotsETL class

        :param data_url: URL to data file
        :param src_dir_path: Path to cancer hotspots data directory
        :param transformed_data_path: Path to transformed cancer hotspots file
        :param ignore_transformed_data: `True` if only bare init is needed. This is
            intended for developers when using the CLI to transform cancer hotspots
            data. Ignores path set in `_transformed_data_path`. `False` will load
            transformed data from s3
        """
        super().__init__(
            data_url, src_dir_path, transformed_data_path, ignore_transformed_data
        )
        fn = self.data_url.split("/")[-1]
        self.data_path = self.src_dir_path / fn
        self.transformed_data = {}  # vrs_id: hotspot data

    def download_data(self) -> None:
        """Download Cancer Hotspots data."""
        if not self.data_path.exists():
            r = requests.get(self.data_url, timeout=5)
            if r.status_code == 200:
                with self.data_path.open("wb") as f:
                    f.write(r.content)
            else:
                _logger.error(
                    "Unable to download Cancer Hotspots data. Received status code: %i",
                    r.status_code,
                )

    async def add_vrs_identifier_to_data(self) -> None:
        """Normalize variations in cancer hotspots and updates `transformed_data`

        Run manually each time variation-normalizer or Cancer Hotspots releases a new
        version.
        """
        self.download_data()
        if not self.data_path.exists():
            err_msg = "Downloading Cancer Hotspots data was unsuccessful"
            raise CancerHotspotsETLError(err_msg)

        snv_hotspots = pd.read_excel(self.data_path, sheet_name="SNV-hotspots")
        indel_hotspots = pd.read_excel(self.data_path, sheet_name="INDEL-hotspots")
        variation_normalizer = QueryHandler()

        _logger.info("Normalizing Cancer Hotspots data...")
        start = timer()
        await self.get_transformed_data(snv_hotspots, variation_normalizer, is_snv=True)
        await self.get_transformed_data(
            indel_hotspots, variation_normalizer, is_snv=False
        )
        end = timer()

        _logger.info("Transformed Cancer Hotspots data in %.*f s", 2, end - start)

        today = datetime.datetime.strftime(
            datetime.datetime.now(tz=datetime.UTC), "%Y%m%d"
        )
        transformed_data_path = self.src_dir_path / f"cancer_hotspots_{today}.json"
        with transformed_data_path.open("w") as f:
            json.dump(self.transformed_data, f)

        _logger.info("Successfully transformed Cancer Hotspots data.")

    async def get_transformed_data(
        self, df: pd.DataFrame, variation_normalizer: QueryHandler, is_snv: bool
    ) -> None:
        """Normalize variant and updates `transformed_data`

        :param df: Dataframe to transform
        :param variation_normalizer: Variation Normalizer handler
        :param is_snv: `True` if SNV data, else INDEL
        """
        for _, row in df.iterrows():
            hugo_symbol = row["Hugo_Symbol"]
            alt = row["Variant_Amino_Acid"]
            pos = row["Amino_Acid_Position"]

            if is_snv:
                ref = row["ref"]
                variation = f"{hugo_symbol} {ref}{pos}{alt.split(':')[0]}"
            else:
                ref = None
                variation = f"{hugo_symbol} {alt.split(':')[0]}"

            try:
                variation_norm_resp = (
                    await variation_normalizer.normalize_handler.normalize(variation)
                )
            except Exception as e:
                _logger.error(
                    "variation-normalizer unable to normalize %s: %s", variation, str(e)
                )
            else:
                if variation_norm_resp and variation_norm_resp.variation:
                    vrs_id = variation_norm_resp.variation.id
                    if vrs_id in self.transformed_data:
                        _logger.debug(
                            "duplicate vrs_id (%s) for variation (%s)",
                            vrs_id,
                            variation,
                        )

                    mutation, observations = alt.split(":")

                    if is_snv:
                        codon = f"{ref}{pos}"
                        mutation = f"{codon}{mutation}"
                    else:
                        codon = pos

                    self.transformed_data[vrs_id] = {
                        "variation": variation,
                        "codon": codon,
                        "mutation": mutation,
                        "q_value": float(row["qvalue"]),
                        "observations": int(observations),
                        "total_observations": int(row["Mutation_Count"]),
                    }
                else:
                    _logger.warning(
                        "variation-normalizer unable to normalize: %s", variation
                    )
